' vbc /r:System.dll,System.Xml.dll Lucifer.vb

Option Strict On

Imports System
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.IO
Imports System.Xml
Imports Microsoft.VisualBasic

Namespace Lucifer

	Public Module MStart


#Region "Fields"
		Private s_ConfigDoc As XmlDocument
		Private s_Solutions As IList(Of CSolutionInfo)
		Private s_BaseDir As String
		Private s_OutputDir As String
		Private s_BuildBatch As String
#End Region


#Region "Entry"

		Public Sub Main(ByVal args() As String)

			ShowHello()

			If args.Length < 2 Then
				ShowSyntaxAndExit()
			End If

			LoadConfig(args(0))

			If args.Length = 2 AndAlso args(1) = "/config" Then
				ShowConfig()
			ElseIf args.Length = 2 AndAlso args(1) = "/all" Then
				For Each solution As CSolutionInfo In s_Solutions
					Process(solution)
				Next solution
			ElseIf args.Length = 2 AndAlso args(1).StartsWith("/name:") Then
				Dim sName As String = args(1).Substring(6)
				sName = TranslateSolutionName(sName)
				If CheckSolutionName(sName) Then
					Dim solution As CSolutionInfo = GetSolution(sName)
					Process(solution)
				Else
					ShowSyntaxAndExit()
				End If
			Else
				Dim sFrom As String = GetOptionValue(args, "/from:")
				Dim sAfter As String = GetOptionValue(args, "/after:")
				Dim sUntil As String = GetOptionValue(args, "/until:")
				Dim sThru As String = GetOptionValue(args, "/thru:")
				sFrom = TranslateSolutionName(sFrom)
				sAfter = TranslateSolutionName(sAfter)
				sUntil = TranslateSolutionName(sUntil)
				sThru = TranslateSolutionName(sThru)
				If CheckSolutionNames(sFrom, sAfter, sUntil, sThru) Then
					Dim fStart, fWillStart As Boolean
					For Each solution As CSolutionInfo In s_Solutions
						' from, after, until throu
						If sFrom = String.Empty AndAlso sAfter = String.Empty Then
							fStart = True
						ElseIf fWillStart Then
							fStart = True
						ElseIf solution.RelPath = sFrom Then
							fStart = True
						ElseIf solution.RelPath = sAfter Then
							fWillStart = True
						End If
						' until
						If solution.RelPath = sUntil Then
							Exit For
						End If
						' so
						If fStart Then
							Process(solution)
						End If
						' thru
						If solution.RelPath = sThru Then
							Exit For
						End If
					Next solution
				Else
					ShowSyntaxAndExit()
				End If
			End If

		End Sub

		Private Sub ShowHello()
			Console.WriteLine("Lucifer - .NET Solution Builder")
		End Sub

		Private Sub ShowConfig()
			Dim xn As XmlNode, xeRoot As XmlElement = s_ConfigDoc.DocumentElement
			Console.WriteLine("Configuration:")
			Dim iAssembly As Int32 = 0
			For Each xn In xeRoot.ChildNodes
				Select Case xn.Name
					Case "basedir"
						Console.WriteLine("  base directory <" & s_BaseDir & ">")
					Case "outputdir"
						Console.WriteLine("  output directory <" & s_OutputDir & ">")
					Case "solution"
						Dim s As String = xn.Attributes("relpath").Value
						Console.WriteLine("  solution(" & iAssembly.ToString() & ") <" & s & ">")
						iAssembly += 1
				End Select
			Next xn
		End Sub

		Private Sub ShowSyntaxAndExit()
			Console.WriteLine("Syntax:")
			Console.WriteLine("  Lucifer <config-file> /config")
			Console.WriteLine("  Lucifer <config-file> /all")
			Console.WriteLine("  Lucifer <config-file> /name:<name-or-index>")
			Console.WriteLine("  Lucifer <config-file> /from:<name-or-index> | /after:<name-or-index> | /until:<name-or-index> | /thru:<name-or-index>")
			Console.WriteLine("  (run Lucifer <config-file> /config to show a list of solution names)")
			Environment.Exit(1)
		End Sub

		Private Sub LoadConfig(ByVal sConfigFile As String)
			s_Solutions = New List(Of CSolutionInfo)
			s_ConfigDoc = New XmlDocument()
			s_ConfigDoc.Load(sConfigFile)
			Dim xeRoot As XmlElement = s_ConfigDoc.DocumentElement
			For Each xn As XmlNode In xeRoot.ChildNodes
				Select Case xn.Name
					Case "basedir"
						s_BaseDir = xn.Attributes("fullpath").Value
					Case "outputdir"
						s_OutputDir = xn.Attributes("relpath").Value
					Case "buildbatch"
						s_BuildBatch = xn.Attributes("filename").Value
					Case "solution"
						Dim s As String = xn.Attributes("relpath").Value
						Dim solution As New CSolutionInfo(s)
						s_Solutions.Add(solution)
						For Each xeSolutionChild As XmlElement In xn.ChildNodes
							If xeSolutionChild.Name = "binary" Then
								Dim sBinPath As String = xeSolutionChild.Attributes("relpath").Value
								solution.Binaries.Add(sBinPath)
							ElseIf xeSolutionChild.Name = "buildbatch" Then
								Dim sBuildBatch As String = xeSolutionChild.Attributes("filename").Value
								solution.BuildBatch = sBuildBatch
							End If
						Next xeSolutionChild
				End Select
			Next xn
		End Sub

		Private Function GetOptionValue(ByVal args() As String, ByVal sOptionName As String) As String
			Dim s As String, sRet As String = Nothing
			For Each s In args
				If s.StartsWith(sOptionName) Then
					Dim iColon As Int32 = sOptionName.IndexOf(":")
					If iColon >= 0 Then
						sRet = s.Substring(iColon + 1)
						Exit For
					End If
				End If
			Next s
			Return sRet
		End Function

		Private Function TranslateSolutionName(ByVal sNameOrIndex As String) As String
			If Information.IsNumeric(sNameOrIndex) Then
				Try
					Dim index As Int32 = CInt(sNameOrIndex)
					Return GetSolution(index).RelPath
				Catch ex As Exception
					Console.WriteLine(ex.Message)
					ShowSyntaxAndExit()
					Return Nothing
				End Try
			Else
				Return sNameOrIndex
			End If
		End Function

		Private Function CheckSolutionNames(ByVal ParamArray strings() As String) As Boolean
			Dim s As String, fRet As Boolean
			For Each s In strings
				If s <> String.Empty Then
					fRet = True
					If Not CheckSolutionName(s) Then
						fRet = False
						Exit For
					End If
				End If
			Next s
			Return fRet
		End Function

		Private Function CheckSolutionName(ByVal sName As String) As Boolean
			Dim index As Integer = GetSolutionIndex(sName)
			Return (index >= 0)
		End Function

		Private Function GetSolution(ByVal index As Int32) As CSolutionInfo
			If index < 0 OrElse index >= s_Solutions.Count Then
				Throw New ArgumentException("Solution index '" & index & "'of range.")
			End If
			Return s_Solutions.Item(index)
		End Function

		Private Function GetSolution(ByVal sName As String) As CSolutionInfo
			Dim index As Integer = GetSolutionIndex(sName)
			If index < 0 Then
				Throw New ArgumentException("Solution '" & sName & "' not found.")
			End If
			Return GetSolution(index)
		End Function

		Private Function GetSolutionIndex(ByVal sName As String) As Integer
			Dim i As Integer = -1
			For Each solution As CSolutionInfo In s_Solutions
				i += 1
				If solution.RelPath = sName Then
					Return i
				End If
			Next solution
			Return -1
		End Function

#End Region


#Region "Processing"

		Private Sub Process(ByVal solution As CSolutionInfo)
			Build(solution)
			Copy(solution)
		End Sub

		Private Sub Build(ByVal solution As CSolutionInfo)
			Dim sSolutionFile As String = Path.Combine(s_BaseDir, solution.SolutionFile)
			Console.Write("Building " & solution.RelPath & " ...")
			If solution.BuildBatch IsNot Nothing AndAlso solution.BuildBatch.Length > 0 Then
				Shell(solution.BuildBatch & " " & Chr(34) & sSolutionFile & Chr(34), , True)
			Else
				Shell(s_BuildBatch & " " & Chr(34) & sSolutionFile & Chr(34), , True)
			End If
			Console.WriteLine(" Done")
		End Sub

		Private Sub Copy(ByVal solution As CSolutionInfo)
			Dim sSolutionDir As String = Path.Combine(s_BaseDir, solution.RelPath)
			For Each sBinPath As String In solution.Binaries
				Dim sFullBinPath As String = Path.Combine(sSolutionDir, sBinPath)
				Copy(sFullBinPath)
			Next sBinPath
		End Sub

		Private Sub Copy(ByVal sSourcePath As String)
			Dim sTargetDir As String = Path.Combine(s_BaseDir, s_OutputDir)
			Dim sFileName As String = Path.GetFileName(sSourcePath)
			Dim sTargetPath As String = Path.Combine(sTargetDir, sFileName)
			If File.Exists(sSourcePath) Then
				Console.Write("Copying " & sSourcePath & " ...")
				File.Copy(sSourcePath, sTargetPath, True)
				Console.WriteLine(" Done")
			Else
				Console.WriteLine("ERROR: File '" & sSourcePath & "' not found.")
			End If
		End Sub

#End Region


	End Module

End Namespace